/* cnfsel.c */

static char rcsid[] = "$Id: cnfsel.c,v 1.1 1994/07/31 15:40:43 JACK Exp $";

#include	<string.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/dos.h>
#include	<sys/iocs.h>
#include	<sys/xglob.h>

#include	"cnfsel.h"
#include	"version.h"
#include	"joy_stick.h"

/* ϐ SLASH ̒l */
char SLASH;

/* XCb`w */
int opt_d = 0;
int opt_k = 0;
int opt_r = 0;
int verbose = 0;

/* ύXÕu[N`FbN̒l */
int break_flag = -1;

/* g̖O */
char *progname;

/* XbVݒQƂ */
static void set_slash(void)
{
	char *p;

	if ((p = getenv("SLASH")) != NULL)
		SLASH = *p;
	else
		SLASH = '\\';
}

/* XbV𓝈ꂷ */
void toslash(char *str)
{
	if (SLASH == '/')
		_toslash(str);
	else
		_tobslash(str);
}

/* x[Xl[߂ */
char *get_basename(char *fname)
{
	char *p;

	if ((p = strrchr(fname, ':')) != NULL)	fname = p + 1;
	if ((p = strrchr(fname, '/')) != NULL)	fname = p + 1;
	if ((p = strrchr(fname, '\\')) != NULL)	fname = p + 1;
	return fname;
}

/* gqׂ */
char *suppress_extension(char *fname)
{
	int len;

	/* .x .X .r .R .z .Z ׂ */
	if ((len = strlen(fname)) > 2) {
		if (fname[len-2] == '.') {
			switch (fname[len-1]) {
				case 'x':
				case 'X':
				case 'r':
				case 'R':
				case 'z':
				case 'Z':
					fname[len-2] = '\0'; break;
			}
		}
	}
	return fname;
}

/* u[N`FbN̐ݒ߂ */
void reset_breakcheck(void)
{
	if (break_flag != -1) {
		_dos_breakck(break_flag);
		break_flag = -1;
	}
}

/* u[N`FbN̐ݒ OFF ɂ */
void set_breakcheck(void)
{
	atexit(reset_breakcheck);
	break_flag = _dos_breakck(-1);
	_dos_breakck(0);
}

void *xmalloc(size_t size)
{
	void *p;

	if (p = malloc(size))	return p;
	perror(NULL);
	exit(EXIT_FAILURE);
}

void xfree(void *p)
{
	if (p)	free(p);
}

char *xstrdup(char *str)
{
	char *p;

	if (p = strdup(str))	return p;
	perror(NULL);
	exit(EXIT_FAILURE);
}

long str2long(char *str, char **endptr)
{
	int radix = 0;

	if (str[0] == '$') {
		str++;
		radix = 16;
	} else if (str[0] == '0' && (str[1] == 'b' || str[1] == 'B')) {
		str += 2;
		radix = 2;
	}
	return strtol(str, endptr, radix);
}

void version(void)
{
	fprintf(stderr, "%s %s (c) JACK\n", progname, VERSION);
}

void usage(void)
{
	fprintf(stderr, "usage: %s [-drvhV] [-f file] [-k key] [-m menu] file\n", progname);
}

void help(void)
{
	static char *help_message[] = {
		" -f file   set temporary file",
		" -k key    open menu selector",
		" -m menu   selector menu list",
		" -d        delete temporary file",
		" -r        reset system",
		" -v        verbose mode",
		" -h        give this help",
		" -V        display version number",
		NULL
	};
	char **p = help_message;

	version();
	usage();
	while(*p)	fprintf(stderr, "%s\n", *p++);
}

int write_tmp(const char *tmpfile, const char *target)
{
	FILE *ifp;
	FILE *ofp;
	char buf[MAX_LINE_SIZE+1];

	if ((ifp = fopen(target, "rt")) == NULL) {
		if (verbose) {
			fprintf(stderr, "%s: t@C %s ܂.\n", progname, target);
		}
		exit(EXIT_ERROR);
	}
	if ((ofp = fopen(tmpfile, "wt")) == NULL) {
		if (verbose) {
			fprintf(stderr, "%s: t@C %s ɏo܂.\n", progname, tmpfile);
		}
		fclose(ifp);
		exit(EXIT_ERROR);
	}

	while (! feof(ifp)) {
		fgets(buf, MAX_LINE_SIZE, ifp);
		fputs(buf, ofp);
	}

	if (ferror(ifp) | ferror(ofp)) {
		if (verbose) {
			if (ferror(ifp)) {
				fprintf(stderr, "%s: t@C %s ǂݏo܂.\n", progname, target);
			} else if (ferror(ofp)) {
				fprintf(stderr, "%s: t@C %s ɏo܂.\n", progname, tmpfile);
			}
		}
		fclose(ifp);
		fclose(ofp);
		exit(EXIT_ERROR);
	}

	fclose(ifp);
	fclose(ofp);
	return 0;
}

int write_name(const char *tmpfile, const char *target)
{
	FILE *ofp;

	if ((ofp = fopen(tmpfile, "wt")) == NULL) {
		if (verbose) {
			fprintf(stderr, "%s: t@C %s ɏo܂.\n", progname, tmpfile);
		}
		exit(EXIT_ERROR);
	}

	fputs(target, ofp);

	if (ferror(ofp)) {
		if (verbose) {
			fprintf(stderr, "%s: t@C %s ɏo܂.\n", progname, tmpfile);
		}
		fclose(ofp);
		exit(EXIT_ERROR);
	}

	fclose(ofp);
	return 0;
}

int get_target(const char *tmp, char *target, int max)
{
	FILE *ifp;
	char buf[MAX_LINE_SIZE+1];

	if ((ifp = fopen(tmp, "rt")) == NULL) {
		if (verbose) {
			fprintf(stderr, "%s: t@C %s ܂.\n", progname, tmp);
		}
		return 0;
	}

	fgets(buf, MAX_LINE_SIZE, ifp);

	if (ferror(ifp)) {
		if (verbose) {
			fprintf(stderr, "%s: t@C %s ǂݏo܂.\n", progname, tmp);
		}
		fclose(ifp);
		return 0;
	}

	strncpy(target, buf, max);
	fclose(ifp);
	return 1;
}

volatile void reset(void)
{
	/* \tgEFAZbg */
	asm (" clr.l d0\n"
		 " trap #10");
}

int get_key_code(char *arg)
{
	struct key_code {
		char *name;
		int code;
	};
	struct key_code key_code_table[] = { "shift", 0x0001,
										 "ctrl",  0x0002,
										 "opt1",  0x0004,
										 "opt2",  0x0008,
										 "joy1a", 0x0010,
										 "joy1b", 0x0020,
										 "joy2a", 0x0040,
										 "joy2b", 0x0080,
										 "mousel",0x0100,
										 "mouser",0x0200,
										 NULL, 0
	};
	int i;
	int code = 0;
	int first = 0;
	char *dlm = "\t ,";
	char *tp;

	while ((tp = strtok((first++ ? NULL: arg), dlm)) != NULL) {
		for (i = 0; key_code_table[i].name != NULL; i++) {
			if (strcmp(key_code_table[i].name, tp) == 0) {
				code |= key_code_table[i].code;
				break;
			}
		}
	}
	return code;
}

void cursor_on(void)
{
	_iocs_b_curon();
	_iocs_skey_mod(-1, 0, 0);
}

int main(int argc, char **argv)
{
	static const char *tmpfile = "\\CONFIG.TMP";
	char cnfsel_tmpfile[MAX_PATH_LEN+1] = "\\sys\\cnfsel.tmp";
	char menu_file[MAX_PATH_LEN+1] = "\\sys\\cnfsel.lst";
	char target[MAX_PATH_LEN+1];
	char *selected_file;
	int ch;

	/* vO̐ݒ */
	progname = xstrdup(get_basename(argv[0]));
#ifdef	SUPPRESS_EXTENSION
	suppress_extension(progname);
#endif

	/* XbVݒ */
	set_slash();

	/* u[N`FbN̐ݒ */
	set_breakcheck();

	/* J[\̈ꎞ}EXJ[\ */
	atexit(cursor_on);
	_iocs_b_curoff();
	_iocs_skey_mod(0, 0, 0);
	_iocs_ms_curof();

	while ((ch = getopt(argc, argv, "f:k:m:drvhV")) != EOF) {
		switch (ch) {
			case 'f':
				strncpy(cnfsel_tmpfile, optarg, PATH_MAX); break;
			case 'k':
				opt_k = get_key_code(optarg); break;
			case 'm':
				strncpy(menu_file, optarg, PATH_MAX); break;
			case 'd':
				opt_d = 1; break;
			case 'r':
				opt_r = 1; break;
			case 'v':
				verbose = 1; break;
			case 'h':
				help(); exit(EXIT_ERROR); break;
			case 'V':
				version(); exit(EXIT_OK); break;
			default:
				usage(); exit(EXIT_ERROR); break;
		}
	}

#ifdef TEST
	if (optind >= argc) {
		ch = 0;
	} else {
		ch = str2long(argv[optind], NULL);
		switch (ch) {
			case 1:
				setup_joy_stick(ch, 1, 20, 3); break;
			case 2:
				setup_joy_stick(ch, 1, 20, 3); break;
			case 3:
				_iocs_ms_init(); break;
				break;
		}
	}
	selected_file = selector(ch, menu_file);
	exit(EXIT_OK);
#endif

	if (optind >= argc) {
		if (opt_r)	reset();

		/* j[NȁH */
		if (opt_k) {
			if ((opt_k & 0x000f) && (_iocs_b_sftsns() & opt_k)) {
				/* L[{[h */
				ch = 0;
			} else if ((opt_k & 0x0030) && (((~_iocs_joyget(0) >> 1) & 0x30) & opt_k)) {
				/* WCXeBbNP */
				ch = 1;
				setup_joy_stick(ch, 1, 20, 3);
			} else if ((opt_k & 0x00c0) && (((~_iocs_joyget(1) << 1) & 0xc0) & opt_k)) {
				/* WCXeBbNQ */
				ch = 2;
				setup_joy_stick(ch, 1, 20, 3);
			} else if ((opt_k & 0x0300) && ((_iocs_ms_getdt() & 0x0100) & opt_k
					|| ((_iocs_ms_getdt() << 8) & 0x0200) & opt_k)) {
				/* }EX */
				ch = 3;
				_iocs_ms_init();
			} else {
				/* ĂȂ */
				ch = -1;
			}
			if (ch >= 0) {
				selected_file = selector(ch, menu_file);
				if (selected_file != NULL) {
					/* CONFIG.TMP N */
					ch = write_tmp(tmpfile, selected_file);
				} else {
					/* Ȃ */
					ch = EXIT_ERROR;
				}
				if (opt_d)	remove(cnfsel_tmpfile);
				exit(ch);
			}
		}

		/* ^[QbgĂяo */
		if (get_target(cnfsel_tmpfile, target, PATH_MAX) == 0) {
			if (opt_d)	remove(cnfsel_tmpfile);
			exit(EXIT_ERROR);
		} else {
			/* EXCONFIGł̓ */
			ch = write_tmp(tmpfile, target);
			if (opt_d)	remove(cnfsel_tmpfile);
			exit(ch);
		}
	}

	/* ƃt@CɃ^[Qbgt@Co */
	strncpy(target, argv[optind], PATH_MAX);
	write_name(cnfsel_tmpfile, target);
	if (opt_r)	reset();
	return 0;
}
