/* joy_stick.c */

static char rcsid[] = "$Id: joy_stick.c,v 1.2 1994/07/31 12:39:13 JACK Exp $";

#include	<sys/iocs.h>
#include	<unistd.h>
#include	"joy_stick.h"

static int joy_time[2] = { 0, 0 };
static int joy_repeat[2] = { -1, -1 };
/* repeat level = 0:no repeat 1:stick 2:trigger 3:stick & trigger */
static int joy_level[2] = { 0, 0 };
static int joy_old_state[2] = { -1, -1 };
static int joy_old_data[2] = { -1, -1 };
static int joy_table[16] = { 0, 1, 5, -1, 7, 8, 6, -1, 3, 2, 4, -1, -1, -1, -1, -1 };
/* ŏ̃s[gJn܂ł̃EFCg */
static int FIRST_REPEAT_TIME[2] = { 20, 20 };	/* 200ms */
/* Qڈȍ~̃EFCg */
static int NEXT_REPEAT_TIME[2] = { 3, 3 };		/*  30ms */

/* [NAs[gxAs[gԊuȂǂݒ肷 */
void setup_joy_stick(int joy_no, int level, int first_time, int next_time)
{
	if (joy_no != 1 && joy_no != 2) return;
	joy_no--;
	if (level > 3) return;

	joy_time[joy_no] = 0;
	joy_repeat[joy_no] = -1;
	joy_old_state[joy_no] = -1;
	joy_old_data[joy_no] = -1;

	if (level >= 0 ) joy_level[joy_no] = level;
	if (first_time >= 0) FIRST_REPEAT_TIME[joy_no] = first_time;
	if (next_time >= 0) NEXT_REPEAT_TIME[joy_no] = next_time;
}

static int check_data(int joy_no, int data)
{
	int time, now;

	now = _iocs_ontime();
	if (data != joy_old_data[joy_no]) {
		joy_old_data[joy_no] = data;
		joy_repeat[joy_no]++;
		if (joy_repeat[joy_no] == 0) {
			joy_time[joy_no] = now;
		}
	} else {
		data = 0;
		time = now - joy_time[joy_no];
		if (time < 0)	time += 24*60*60*100;
		if (joy_repeat[joy_no] == 0) {
			time -= FIRST_REPEAT_TIME[joy_no];
			if (time >= 0) {
				joy_old_data[joy_no] = -1;
				joy_time[joy_no] = now + time;
			}
		} else {
			time -= NEXT_REPEAT_TIME[joy_no];
			if (time >= 0) {
				joy_old_data[joy_no] = -1;
				joy_time[joy_no] = now + time;
			}
		}
	}
	return data;
}

int get_joy_stick(int joy_no)
{
	int c0, c1, c2, c3;
	int c, cc, ct;

	if (joy_no != 1 && joy_no != 2) return 0;
	joy_no--;

	/* cc = cursor no., ct = trigger */
	c0 = _iocs_joyget(joy_no);
	c = ~c0;
	ct = c & 0x60;
	cc = joy_table[c & 0x0f];

	c1 = _iocs_joyget(joy_no);
	c2 = _iocs_joyget(joy_no);
	c3 = _iocs_joyget(joy_no);
	if (c0 != c1 || c0 != c2 || c0 != c3) {
			return 0;
	}

	if (((~c0) & 0xff) == 0) {
		/* ͂ȂΏċA */
		joy_repeat[joy_no] = -1;
		joy_old_state[joy_no] = 0;
		return c;
	}

	switch (joy_level[joy_no]) {
		case 0:
			c = (~c0) & 0xff;
			c = ((joy_old_state[joy_no] ^ c) & c);
			joy_old_state[joy_no] = (~c0) & 0xff;
			cc = joy_table[c & 0x0f];
			ct = c & 0xf0;
			c = cc | ct;
			break;
		case 1:
			c = ct;
			c = ((joy_old_state[joy_no] ^ c) & c);
			joy_old_state[joy_no] = ct;
			ct = c;

			cc = check_data(joy_no, cc);
			c = cc | ct;
			break;
		case 2:
			c = (~c0) & 0x0f;
			c = ((joy_old_state[joy_no] ^ c) & c);
			joy_old_state[joy_no] = (~c0) & 0x0f;
			cc = joy_table[c & 0x0f];

			ct = check_data(joy_no, ct);
			c = cc | ct;
			break;
		case 3:
			c = check_data(joy_no, (cc | ct));
			break;
	}
	return c;
}

#ifdef TEST
#include	<conio.h>
#include	<stdio.h>
#include	<stdlib.h>

void main(int argc, char **argv)
{
	int c;

	if (argc > 1) {
		c = atoi(argv[1]);
		setup_joy_stick(1, c, -1, -1);
	}

	while (1) {
		if (kbhit() && (getch() == 0x1b)) break;
		c = get_joy_stick(1);
		c &= 0xff;
		switch (c & 0x0f) {
			case JOY_UP:	printf(" "); break;
			case JOY_RUP:	printf(" E"); break;
			case JOY_RIGHT:	printf(" E"); break;
			case JOY_RDOWN:	printf(" E"); break;
			case JOY_DOWN:	printf(" "); break;
			case JOY_LDOWN:	printf(" "); break;
			case JOY_LEFT:	printf(" "); break;
			case JOY_LUP:	printf(" "); break;
			case JOY_NULL:
			default:		break;
		}
		if (c & JOY_A)		printf(" `");
		if (c & JOY_B)		printf(" a");
		fflush(stdout);
	}
}
#endif
