/*
 *	sps.c	(sps)
 *		super process
 *
 *		by ici	Dec. 4, 1989
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/iocs.h>
#include <sys/xstart.h>

#include "sps.h"

/* External functions */
extern void get_bgthread_table (void);		/* spsb.c */
extern void get_process_table (void);		/* spsp.c */
extern void get_sram_boot_program (void);	/* spsp.c */
extern void get_device_driver (void);		/* spsd.c */
extern int search_and_print_vector (int, long, long, int);	/* spsvec.c */
extern void print_device (struct Device *);	/* spsd.c */
extern int psnamecheck (char*, struct Process*);	/* spsp.c */
extern void print_process (struct Process *);	/* spsp.c */
extern void print_bg (struct BG *);		/* spsb.c */
extern void print_timer (void);			/* spst.c */
extern void print_memory (void);		/* spsm.c */

/* Variables */
struct Process	process[MAX_P];
struct Device	device[MAX_D];
struct BG	bg[MAX_B];
int		n_process;
int		n_device;
int		n_bg;
struct Mcb*	top_mcb;
long		memory_size;

char* status_name[] = {
	"PROC  ",	/* 0 */
	"KEEP  ",	/* 1 */
	"MEMDRV",	/* 2 */
	"CHAR  ",	/* 3 */
	"BLOCK ",	/* 4 */
	"HUMAN ",	/* 5 */
	"SRAM  ",	/* 6 */
	"SPBLCK",	/* 7 */
};

/* option flags */
int aflag;
int bflag;
int vflag;
int dflag;
int pflag;
int mflag;
int tflag;
int slashflag;
char* processname;

/* File scope variables */
static const char version[] = "version 1.00 patchlevel 2";

/* File scope functions */
static void volatile usage (int code);

/* File scope functions */
static int
dvnamecheck (char* name, struct Device* d)
{
    int len = strlen (name);

    if (d->status == P_CHAR)
	return !strncasecmp (name, d->name, len);
    else
	return (!strncasecmp (name, d->name, len)
	     || !strncasecmp (name, d->name + 1, len));
}

/* File scope functions */
static int
sps (const char* myname)
{
    char buf[300];
    struct Process* p;
    struct Device* d;
    struct BG* b;
    int i;
    int sps_size;
    int exitvalue;
    int process_size = process[0].start;

    if (pflag || dflag) {
	/*	 00000000 00000000  00000+000000 (99%) 0000 PROC   sps al */
	printf ("START___ END_____  SIZE______________ MALC STATUS %s%s%s\n",
			dflag ? "DEVICE" : "",
			pflag && dflag ? " / " : "",
			pflag ? "COMMAND" : "");
    }

    /* IPL PROGRAM AREA */
    if (vflag && search_and_print_vector (NOPRINT, 0x1000, 0x67ff, FALSE)) {
	search_and_print_vector (VECPRINT, 0x1000, 0x67ff, FALSE);
    }

    /* for n option */
    exitvalue = processname ? 2 : 0;

    /* print devices */
    if (dflag) {
	for (i = 0, d = device; i < n_device; i++, d++) {
	    if (!(processname == NULL || dvnamecheck (processname, d)))
		continue;
	    exitvalue = 0;
	    print_device (d);
	}
    }
    if (pflag) {
	/* print processes */
	for (i = 0, p = process; i < n_process; i++, p++) {
	    if (!(processname == NULL || psnamecheck (processname, p)))
		continue;
	    exitvalue = 0;
	    print_process (p);
	}
    }
    if (bflag) {
	for (i = 0, b = bg; i < n_bg; i++, b++) {
	    if (!pflag || b->process == 0)
		print_bg (b);
	}
    }

    sps_size = 0;
    for (i = 0, p = process; i < n_process; i++, p++) {
	strcat (strcpy (buf, p->name - 0x44), p->name);
	if (strcmpi (buf, myname) == 0)
	    sps_size = process[i].size + process[i].mem;
	else
	    process_size += process[i].size + process[i].mem;
    }
    if (*(unsigned char*)0xcbc == 6) {
	register int_ reg_d0 __asm ("d0");

	__asm __volatile ("move #$c002,d1\n\t"
			  "moveq #__SYS_STAT,%0\n\t"
			  "trap #15"
			  : "=d" (reg_d0) : : "d1");

	if (reg_d0) {
	    process_size -= (0x10000000 - 0x00f00000);
	    memory_size  -= (0x10000000 - 0x00f00000);
	}
    }

    if (tflag)
	print_timer ();
    if (mflag)
	print_memory ();
    if (pflag || dflag || tflag || mflag)
	printf ("\n");

    i = memory_size - process_size;
    printf ("  %d (%dKB) process, %d device, %d thread, %dKB (%2d%%) free.\n",
		n_process,
		KBYTE (process_size + sps_size),
		n_device,
		n_bg,
		(int) KBYTE (i),
#if 0
		(int) ((i) * 100 / memory_size)
#else
		(int) ((i) * 100U / memory_size)
#endif
	);

    return exitvalue;
}

/* File scope functions */
static void
analyzeoption (int argc, char* argv[])
{
    int i;

    pflag = TRUE;

    for (i = 1; i < argc; i++) {
	char* p = argv[i];
	char c;

	if (*p == (char)'-') {
	    p++;
	    if (strcmp (p, "-help") == 0) {
		usage (0);
		/* NOT REACHED */
	    }
	    if (strcmp (p, "-version") == 0) {
		printf ("sps %s\n", version);
		exit (0);
	    }
	}
	while ((c = *p++) != '\0') {
	    switch (tolower (c)) {
	    case 'a':
		aflag = TRUE;
		break;
	    case 'b':
		bflag = TRUE;
		break;
	    case 'd':
		dflag = TRUE;
		pflag = FALSE;
		break;
	    case 'm':
		mflag = TRUE;
		break;
	    case 'p':
		pflag = TRUE;
		dflag = FALSE;
		break;
	    case 't':
		tflag = TRUE;
		break;
	    case 'v':
		vflag = TRUE;
		break;
	    case 'n':
		if (*p == '\0') {
		    if ((p = argv[++i]) == NULL) {
			usage (1);
			/* NOT REACHED */
		    }
		}
		processname = p;
		p = "";
		break;
	    case 'h':
	    case '?':
		usage (0);
		/* NOT REACHED */
	    default:
		usage (1);
		/* NOT REACHED */
	    }
	}
    }

    if (processname)
	aflag = TRUE;
    if (aflag) {
	bflag = dflag = pflag = TRUE;
	mflag = tflag = FALSE;
    }
    if (mflag || tflag)
	bflag = dflag = pflag = FALSE;
}

/* Functions */
int
main (int argc, char* argv[])
{
    char myname[70];
    char* p;

    if ((p = getenv ("SLASH")) != NULL && p[0] == (char)'/' && !p[1])
	slashflag = TRUE;

    analyzeoption (argc, argv);
    get_process_table ();
    get_bgthread_table ();
    get_device_driver ();
    get_sram_boot_program ();

    strcat (strcpy (myname, (char*) _memcp + 0x80), (char*) _memcp + 0xc4);
    return sps (myname);
}

/* File scope functions */
static void volatile
usage (int code) {
    printf ("Super ProcesS X680x0 custom %s"
		" by ici (K.M.C), 2002/03/20 Ԃq.\n"
	    "usage: sps [-abdmptv] [-n name]\n"
	    "option:\n"
		"\t-a\tfoCXhCoƃvZX\\n"
		"\t-b\tobNOEhXbh\\n"
		"\t-d\tfoCXhCo\\n"
		"\t-m\tubN\\n"
		"\t-p\tvZX\(ftHg)\n"
		"\t-t\t^C}̎gp󋵂\\n"
		"\t-v\txN^ݒ󋵂\\n"
		"\t-n name\tw肵hCo/vZX̏ڍׂ\\n",
		version
	);
    exit (code);
}

/* EOF */
