/*
 *	spsd.c	(sps)
 *		get device table
 *
 *		by ici	Dec. 4, 1989
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "sps.h"

/* External functions */
extern int search_and_print_vector (int, long, long, int);	/* spsvec.c */

/* File scope functions */
static int
compare_device_address (struct Device* l, struct Device* r)
{
    return (int)l->header - (int)r->header;
}

/* File scope functions */
static void
sort_device (void)
{
    qsort (device, n_device, sizeof (struct Device),
		(int (*)(const void*, const void*)) compare_device_address);
}

/* Functions */
void
get_device_driver (void)
{
    unsigned long* devp;
    struct Device* d;
    unsigned long process0end;
    int i;

    /* search NUL */
    {
	unsigned short* p = (unsigned short*) 0x6800;

	for (;; p++) {
	    if (p[0] == 0x4e55 && p[1] == 0x4c20)	/* 'NUL ' */
		break;
	}
	p -= 0x0e / 2;
	devp = (unsigned long*) p;
    }

    /* get device driver */
    i = 0;
    do {
	device[i].header = (char*) devp;
	devp = (unsigned long*) *devp;
	i++;
    } while ((long) devp != 0xffffffff);
    n_device = i;

    /* sort device */
    sort_device ();

    /* set parameters */
    process0end = process[0].end;
    for (i = 0, d = device; i < n_device; i++, d++) {
	int j;
	char* n;
	char* p = d->header;

	d->start = (long) d->header;
	d->end = (long) (d + 1)->header;
	if (d->end > process0end && d->start < process0end)
	    d->end = process0end;
	else if (i + 1 == n_device && d->start < process0end)
	    d->end = process0end;
	else if (d->start > process0end) {
	    int pid;

	    for (pid = 0; pid < n_process; pid++) {
		if (process[pid].start <= d->start
			&& d->start <= process[pid].end) {
		    d->end = process[pid].end;
		    break;
		}
	    }
	}
	d->size = d->end - d->start;
	d->atr = *(unsigned short *) (p + 4);
	d->status = (*(p + 4) & 0x80 ? P_CHAR : (*(p + 4) & 0x20 ? P_SPBLOCK : P_BLOCK));
	p += 14;
	n = d->name;
	*n++ = (*p < ' ') ? ' ' : *p;
	p++;
	for (j = 1; j < 8 && *p != ' '; j++)
	    *n++ = *p++;
	*n++ = '\0';
    }
}

/* File scope functions */
static char *
device_attribute (unsigned int atr)
{
    static char buf[6];

    buf[0] = (atr & 0x4000 ? 'I' : (atr & 0x0040 ? 'I' : '-'));
    buf[1] = (atr & 0x0020 ? 'R' : 'C');
    buf[2] = (atr & 0x0004 ? 'N' : '-');
    buf[3] = ((atr & 0x0002 || atr & 0x0001) ? 'S' : '-');
    buf[4] = '\0';

    return buf;
}

/* Functions */
void
print_device (struct Device* d)
{
    printf ("%08x %08x  %5d+     0 (%2d%%) %s %s %s\n",
		(unsigned int) d->start,
		(unsigned int) d->end - 1,
		(int) KBYTE (d->size),
		(int) (d->size * 100 / memory_size),
		device_attribute (d->atr),
		STATUS (d->status),
		d->name
	);

    if (vflag)
	search_and_print_vector (VECPRINT, d->start, d->end, FALSE);
}

/* EOF */
