/*
 *	spsp.c	(sps)
 *		get process table
 *
 *		by ici	Dec. 4, 1989
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/xglob.h>
#include <sys/xstart.h>
#include "sps.h"

/* External functions */
extern void print_bg (struct BG*);	/* spsb.c */
extern int search_and_print_vector (int, long, long, int);	/* spsvec.c */

/* Functions */
int
isprocess (struct Mcb* m, int* pid)
{
    int i;
    struct Process* p;

    for (i = 0, p = process; i < n_process; i++, p++) {
	if (p->mcb == m) {
	    *pid = i;
	    return TRUE;
	}
	if (p->mcb == m->parent)
	    *pid = i;
    }

    return FALSE;
}

/* File scope functions */
static int
compare_process_address (struct Process* l, struct Process* r)
{
    return (int)l->start - (int)r->start;
}

/* File scope functions */
static void
sort_process (void)
{
    qsort (process, n_process, sizeof (struct Process),
		(int (*)(const void*, const void*)) compare_process_address);
}

/* Functions */
void
get_process_table (void)
{
    struct Mcb* q;
    struct Process* p;
    int i;
    int pid;
    long tmp;

    /* search processes and top of mcb list */
    q = (struct Mcb *) _memcp;
    i = 0;
    do {
	top_mcb = q;
	process[i].status = P_NORMAL;
	process[i++].mcb = q;
	q = q->parent;
    } while (q != 0);

    /* search KEEP and MEMDRV */
    for (q = top_mcb; q != 0; q = q->next) {
	tmp = (long) q->parent;
	tmp = (tmp >> 24) & 0xff;
	if (tmp == 0xff || tmp == 0xfe) {
	    /* KEEP or MEMDRV process */
	    process[i].status = (tmp == 0xff ? P_KEEP : P_MEMDRV);
	    process[i++].mcb = q;
	}
    }
    n_process = i;

    /* set parameters to process table */
    for (i = 0, p = process; i < n_process; i++, p++) {
	char* cmdname;

	q = p->mcb;
	p->start = (long) ((char *) q + 0x100);
	p->end = q->max;
	p->size = p->end - p->start;
	p->mem = 0;
	p->malloc = 0;
	cmdname = (char *) q;
	p->name = cmdname + 0xc4;
	p->bg = NULL;
    }

    /* sort in order of START address */
    sort_process ();
    process[0].status = P_HUMAN;

    /* calculate size */
    for (q = top_mcb; q != 0; q = q->next) {
	if (!isprocess (q, &pid)) {
	    process[pid].mem += (long) q->max - ((long) q + 0x10);
	    process[pid].malloc++;
	}
    }

    /* memory size */
    memory_size = *(long*) 0x1c00;
}

/* Functions */
void
get_sram_boot_program (void)
{
    struct Process* p;
    long* boot = (long*) 0xed0010;
    char* c = (char*) *boot;
    int i;

    if (*c != 0x60 && !search_and_print_vector (NOPRINT, 0xed0000, 0xed4000, FALSE))
	return;

    p = &process[n_process++];
    p->mcb = (struct Mcb*) (p->start = *boot);
    for (i = 0; i < n_device; i++) {
	if (!strcmp (&device[i].name[1], "S_RAM  ")) {
	    p->end = 0xed03ff;
	    goto next;
	}
    }
    p->end = 0xed4000;

next:
    p->size = 0;
    p->mem = 0;
    p->malloc = 0;
    p->status = P_SRAM;
}

/* Functions */
int
psnamecheck (char* p, struct Process* q)
{
    char buf[128];

    strcpy (buf, q->name);
    if (!strchr (p, '.')) {
	char* e = strrchr (buf, '.');
	if (e)
	    *e = '\0';
    }

#if 0
    return !strcasecmp (buf, p);
#else
    return !strcmp (buf, p);
#endif
}

/* Functions */
void
print_process (struct Process* p)
{
    char buf[300];

    printf ("%08x %08x  %5d+%6d (%2d%%) %4d %s ",
		(unsigned int) p->mcb,
		(unsigned int) p->end - 1,
		(int) KBYTE (p->size),
		(int) KBYTE (p->mem),
		(int) ((p->size + p->mem) * 100 / memory_size),
		p->malloc,
		STATUS(p->status)
	);

    if (p->status == P_SRAM) {
	printf ("SRAM BOOT PROGRAM $%08x\n", (unsigned int) p->start);
	if (vflag)
	    search_and_print_vector (VECPRINT, 0xed0000, p->end, FALSE);
	return;
    }
    else if (p->status == P_HUMAN) {
	int vernum = _dos_vernum ();

	printf ("Human68k ver %d.%02d\n", (vernum >> 8) & 0xff, vernum & 0xff);
	if (vflag)
	    search_and_print_vector (VECPRINT, p->start, device[0].start, TRUE);
	return;
    }

    strcat (strcpy (buf, p->name - 0x44), p->name);
    if (slashflag)
	_toslash (buf);

    if (p->status == P_NORMAL) {
	char* cmdline;

	cmdline = *(char**) ((char*) p->mcb + 0x20);
	if (*cmdline)
	    strcat (strcat (buf, " "), cmdline + 1);
    }

    printf ("%s\n", buf);

#if 0	/* Xbh𕡐ĂĂŌ̈\łȂ */
    if (bflag && p->bg)
	print_bg (p->bg);
#endif
    if (bflag) {
	int i;

	for (i = 0; i < MAX_B; ++i) {
	    if (bg[i].process && (bg[i].mcb == p->mcb))
		print_bg (&bg[i]);
	}
    }

    if (vflag) {
	if (p->status == P_HUMAN)
	    search_and_print_vector (VECPRINT, p->start, device[0].start, TRUE);
	else
	    search_and_print_vector (VECPRINT, p->start, p->end, FALSE);
    }
}

/* EOF */
