/*
 *	spst.c	(sps)
 *		timer analyze
 *
 *		by ici Dec. 4, 1989.
 */

#include <stdio.h>
#include <string.h>
#include "sps.h"

/* Macros */
#define	VECTOR(x)		((long*)((x)*4))
#define	MFP_EDGE		((unsigned char *)0xe88003)
#define	MFP_INT_ENABLE_REGA	((unsigned char *)0xe88007)
#define	MFP_INT_ENABLE_REGB	((unsigned char *)0xe88009)
#define	MFP_TIMERA_CONTROL	((unsigned char *)0xe88019)
#define	MFP_TIMERB_CONTROL	((unsigned char *)0xe8801b)
#define	MFP_TIMERCD_CONTROL	((unsigned char *)0xe8801d)
#define	MFP_TIMERA_DATA		((unsigned char *)0xe8801f)
#define	MFP_TIMERB_DATA		((unsigned char *)0xe88021)
#define	MFP_TIMERC_DATA		((unsigned char *)0xe88023)
#define	MFP_TIMERD_DATA		((unsigned char *)0xe88025)

#if 0
static int prescaler[] = {
    0, 4, 10, 16, 50, 64, 100, 200
};
#endif

/* File scope functions */
static int
am (void)
{
    return *MFP_INT_ENABLE_REGA & 0x20;
}
static int
bm (void)
{
    return *MFP_INT_ENABLE_REGA & 0x01;
}
static int
cm (void)
{
    return *MFP_INT_ENABLE_REGB & 0x20;
}
static int
dm (void)
{
    return *MFP_INT_ENABLE_REGB & 0x10;
}
static int
fm (void)
{
    return *MFP_INT_ENABLE_REGB & 0x08;
}
static int
rm (void)
{
    return *MFP_INT_ENABLE_REGB & 0x01;
}

/* File scope variables */
static struct Timer {
    char*	name;
    long*	vector;
    int (*mode) (void);
} timer[] = {
    { "Timer-A (V-DISP)",		VECTOR (0x4d), am },
    { "Timer-B (Keyboard Speed)",	VECTOR (0x48), bm },
    { "Timer-C",			VECTOR (0x45), cm },
    { "Timer-C1(Mouse)",		(long*) 0x9b6, cm },
    { "Timer-C2(Cursor Blink)",		(long*) 0x9be, cm },
    { "Timer-C3(FD Motor On/Off)",	(long*) 0x9c6, cm },
    { "Timer-C4(PowerOff)",		(long*) 0x9ce, cm },
    { "Timer-D (General Purpose)",	VECTOR (0x44), dm },
    { "OPM Timer",			VECTOR (0x43), fm },
    { "RTC Alarm(1Hz/16Hz)",		VECTOR (0x40), rm }
};
#define TIMER_TABLE_NUM (sizeof (timer) / sizeof (timer [0]))

/* File scope functions */
static const char*
search_device (unsigned long v)
{
    struct Device* d;
    int i;

    for (i = 0, d = device; i < n_device; i++, d++) {
	if (d->start <= v && v <= d->end)
	    return d->name;
    }
    return NULL;
}

/* File scope functions */
static const char*
search_process (unsigned long v)
{
    struct Process* p;
    int i;

    for (i = 0, p = process; i < n_device; i++, p++) {
	if (p->start <= v && v <= p->end)
	    return p->name;
    }
    return NULL;
}

/* File scope functions */
static void
vector_what (unsigned long vector)
{
    const char* p;

    if (vector >= 0xfc0000 && vector <= 0xffffff)
	printf ("IOCS ROM");
    else if (vector < process[0].start)
	printf ("HUMAN");
    else if ((p = search_device (vector)) != NULL) {
	printf ("DEVICE %c", *p++);
	while (*p && *p != ' ')
	    printf ("%c", *p++);
    }
    else if ((p = search_process (vector)) != NULL)
	printf ("PROCESS %s", p);
#if 0
    else if (vector > 0xffffff)
	printf ("ERROR");
#endif
    else
	printf ("???");
}

/* Functions */
void
print_timer (void)
{
    int i;
    long himem_flag = get_himem_flag ();

    for (i = 0; i < TIMER_TABLE_NUM; i++) {
	struct Timer* t = &timer[i];
	long vector = *(t->vector);

	if (!(himem_flag & vector))
	    vector &= 0x00ffffff;

	printf ("%-28s %s => %08x (", t->name, (*t->mode) () ? "Enable " : "Disable",
			(unsigned int) vector);
	vector_what (vector);
	printf (")\n");
    }
}

/* EOF */
