/*
 *	spsvec.c	(sps)
 *		interrupt vector search
 *
 *		by ici Dec. 4, 1989.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/iocs.h>

#include "sps.h"

/* Macros */
#define	VECTOR_68000	0
#define	VECTOR_IOCS	1
#define	VECTOR_DOS	2
#define	VECTOR_ETC	3
#define	ATTRIBUTE_NAME(x)	(attribute_name[(x)])
#define	TRUE		1
#define	FALSE		0

/* File scope variables */
static char* attribute_name[] = {
    "CPU ",
    "IOCS",
    "DOS ",
    "ETC "
};

/* File scope variables */
static struct Vector_etctable {
    int*	vector;
    char*	name;
} vector_etctable[] = {
		  /* 0123456789abcdef */
    { (int*) 0x92c, "Mouse Receive" },
    { (int*) 0x934, "SoftKB Manager" },
    { (int*) 0x938, "Mouse Manager" },
    { (int*) 0x978, "Beep Sound Data" },
    { (int*) 0x97e, "ExESC Manager" },
    { (int*) 0x9b6, "Timer-C Mouse" },
    { (int*) 0x9be, "Timer-C CurBlink" },
    { (int*) 0x9c6, "Timer-C FD Motor" },
    { (int*) 0x9ce, "Timer-C PowerOff" },
    { (int*) 0xd12, "Beep Manager" },
    { (int*) 0xd16, "ESCseq Manager" }
};
#define	VECTOR_ETCTABLESIZE	(sizeof (vector_etctable) / sizeof (vector_etctable[0]))

/* File scope functions */
static void print_vector (long*, long, int, int);

/* Functions */
/*	printflag;	if TRUE then print vector information	*/
/*	humanflag;	if HUMAN then don't search in DOS call	*/
int
search_and_print_vector (int printflag, long start, long end, int humanflag)
{
    long* v;
    int i;
    long himem_flag = get_himem_flag ();

    /* 68000 int vector */
    v = 0;
    for (i = 0; i < 256; i++, v++) {
	long a = (himem_flag & *v) ? *v : *v & 0x00ffffff;

	if (start <= a && a <= end) {
	    if (printflag)
		print_vector (v, a, i, VECTOR_68000);
	    else
		return TRUE;
	}
    }

    /* IOCS call */
    v = (long*) 0x400;
    for (i = 0; i < 256; i++, v++) {
	long a = (himem_flag & *v) ? *v : *v & 0x00ffffff;

	if (start <= a && a <= end) {
	    if (printflag)
		print_vector (v, a, i, VECTOR_IOCS);
	    else
		return TRUE;
	}
    }

    /* DOS call */
    if (!humanflag) {
	v = (long*) 0x1800;
	for (i = 0; i < 256; i++, v++) {
	    long a = (himem_flag & *v) ? *v : *v & 0x00ffffff;

	    if (start <= a && a <= end) {
		if (printflag)
		    print_vector (v, a, i, VECTOR_DOS);
		else
		    return TRUE;
	    }
	}
    }

    /* etc. */
    for (i = 0; i < VECTOR_ETCTABLESIZE; i++) {
	long* v = (long*) vector_etctable[i].vector;
	long a = (himem_flag & *v) ? *v : *v & 0x00ffffff;

	if (start <= a && a <= end) {
	    if (printflag)
		print_vector (v, a, i, VECTOR_ETC);
	    else
		return TRUE;
	}
    }

    return FALSE;
}

/* Functions */
long
get_himem_flag (void)
{
    if (*(unsigned char*)0xcbc == 6) {
#if 0
	register int_ reg_d0 __asm ("d0");
	register int_ reg_a1 __asm ("a1");

	__asm __volatile ("move #$8000,d1\n\t"
			  "moveq #__SYS_STAT,%0\n\t"
			  "trap #15"
			  : "=d" (reg_d0), "=a" (reg_a1) : : "d1");

	if (reg_d0 >= 0 && reg_a1 == 0x30363054)	/* '060T' */
#endif
	    return 0x10000000;
    }
    return 0;
}

/* Functions */
void
print_vector (long* vector, long address, int number, int atr)
{
    char vectorname[20];

    if (atr == VECTOR_ETC)
	strcpy (vectorname, vector_etctable[number].name);
    else {
	memcpy (vectorname, vector_list[atr][number], 16);
	vectorname[16] = '\0';
	if (vectorname[0] == '\0')
	    strcpy (vectorname, "???");
    }

    /*	     00000000 00000000  00000+000000 (00%) 0000 PROC   sps al */
    printf ("%51s%s", "", ATTRIBUTE_NAME (atr));

    if (atr == VECTOR_ETC)
	printf ("%04x", (unsigned int) vector);
    else
	printf ("  %02x", number & 0xff);

    printf (" (%s)%*s => %08x\n", vectorname, (int) (16 - strlen (vectorname)), "",
		(unsigned int) address);
}

/* EOF */
